from enum import Enum
from time import sleep
from leg import Leg
from leg import LegName
import smbus
import PCA9685
import ServoPCA9685
import threading
from flask import Flask
from flask import request
from flask_cors import CORS

class Direction(Enum):
    FORWARD=0,
    BACKWARD=3

class DogError(Exception):
    def __init__(self, message):
        self.message = message
    pass

class Dog:
    def __init__(self):
        self.legs = {
                LegName.FORWARD_LEFT  :None,
                LegName.FORWARD_RIGHT :None,
                LegName.BACKWARD_LEFT :None,
                LegName.BACKWARD_RIGHT:None
            }
        self.left_speed = self.right_speed = 1
        pass

    def configure_forward_left_leg(self, pca, channelKnee, channelElbow, channelShoulder):
        leg_name = LegName.FORWARD_LEFT
        reverse =  False
        self.legs[leg_name]=Leg(pca,channelKnee, channelElbow, channelShoulder, reverse, leg_name)
        pass

    def configure_forward_right_leg(self, pca, channelKnee, channelElbow, channelShoulder):
        leg_name = LegName.FORWARD_RIGHT
        reverse = True
        self.legs[leg_name]=Leg(pca,channelKnee, channelElbow, channelShoulder, reverse, leg_name)
        pass

    def configure_backward_left_leg(self, pca, channelKnee, channelElbow, channelShoulder):
        leg_name = LegName.BACKWARD_LEFT
        reverse = False
        self.legs[leg_name]=Leg(pca,channelKnee, channelElbow, channelShoulder, reverse, leg_name)
        pass

    def configure_backward_right_leg(self, pca, channelKnee, channelElbow, channelShoulder):
        leg_name = LegName.BACKWARD_RIGHT
        reverse = True
        self.legs[leg_name]=Leg(pca,channelKnee, channelElbow, channelShoulder, reverse, leg_name)
        pass

    def is_inited(self):
        return self.legs[LegName.FORWARD_LEFT] != None and self.legs[LegName.FORWARD_RIGHT] != None and self.legs[LegName.BACKWARD_LEFT] != None and self.legs[LegName.BACKWARD_RIGHT] != None

    def get_leg_forward_left(self):
        return self.legs[LegName.FORWARD_LEFT]

    def get_leg_forward_right(self):
        return self.legs[LegName.FORWARD_RIGHT]

    def get_leg_backward_left(self):
        return self.legs[LegName.BACKWARD_LEFT]

    def get_leg_backward_right(self):
        return self.legs[LegName.BACKWARD_RIGHT]

    def start(self):
        if not self.is_inited():
            raise DogError('Legs havent initialised')
        leg_forward_left = self.get_leg_forward_left()
        leg_forward_right = self.get_leg_forward_right()
        leg_backward_left = self.get_leg_backward_left()
        leg_backward_right = self.get_leg_backward_right()
        threading.Thread(target = leg_forward_left.main_thread).start()
        threading.Thread(target = leg_forward_right.main_thread).start()
        threading.Thread(target = leg_backward_left.main_thread).start()
        threading.Thread(target = leg_backward_right.main_thread).start()
    
    def set_speds(self, left_speed, right_speed):
        self.left_speed = left_speed
        self.right_speed = right_speed


    def walk(self, steps_count = 16):
        leg_forward_left = self.get_leg_forward_left()
        leg_forward_right = self.get_leg_forward_right()
        leg_backward_left = self.get_leg_backward_left()
        leg_backward_right = self.get_leg_backward_right()

        leg_forward_right.set_speeds(5,2,1)
        leg_backward_left.set_speeds(5,2,1)
        leg_forward_left.set_speeds(5,2,1)
        leg_backward_right.set_speeds(5,2,1)
        time_delay = 0.3
        current_step = 0
        knee_angle = 20
        elbow_angle = 10
        while current_step < steps_count:
            #elbow
            elbow_center = -50
            elbowForwardAngleRight = elbow_center + elbow_angle * self.right_speed
            elbowForwardAngleLeft = elbow_center + elbow_angle * self.left_speed
            elbowBackAngleRight = elbow_center - elbow_angle * self.right_speed
            elbowBackAngleLeft = elbow_center - elbow_angle * self.left_speed
            #knee
            knee_center = 80
        
            kneeBackwardAngleRight = knee_center - knee_angle * self.right_speed
            kneeBackwardAngleLeft = knee_center - knee_angle * self.left_speed
            kneeForwardAngleRight = knee_center + knee_angle * self.right_speed
            kneeForwardAngleLeft = knee_center + knee_angle * self.left_speed

            #step 1
            if self.right_speed >= 0:                  
                leg_forward_right.set_angles(kneeForwardAngleRight, elbowForwardAngleRight, 0)
                leg_backward_right.set_angles(kneeBackwardAngleRight, elbowBackAngleRight, 0)
            else:
                leg_forward_right.set_angles(kneeBackwardAngleRight, elbowForwardAngleRight, 0)
                leg_backward_right.set_angles(kneeForwardAngleLeft, elbowBackAngleLeft, 0)

            if self.left_speed >= 0: 
                leg_backward_left.set_angles(kneeForwardAngleLeft, elbowForwardAngleLeft, 0)
                leg_forward_left.set_angles(kneeBackwardAngleLeft, elbowBackAngleLeft, 0)
            else:
                leg_backward_left.set_angles(kneeBackwardAngleLeft, elbowForwardAngleLeft, 0)
                leg_forward_left.set_angles(kneeForwardAngleRight, elbowBackAngleRight, 0)

            sleep(time_delay)

            #step 2
            if self.right_speed >= 0:
                leg_forward_right.set_angles(kneeBackwardAngleRight, elbowForwardAngleRight, 0)
                leg_backward_right.set_angles(kneeBackwardAngleRight, elbowBackAngleRight, 0)      
            else:
                leg_forward_right.set_angles(kneeForwardAngleRight, elbowForwardAngleRight, 0)
                leg_backward_right.set_angles(kneeBackwardAngleLeft, elbowBackAngleLeft, 0)
            
            if self.left_speed>=0:
                leg_backward_left.set_angles(kneeBackwardAngleLeft, elbowForwardAngleLeft, 0)
                leg_forward_left.set_angles(kneeBackwardAngleLeft, elbowBackAngleLeft, 0)
            else:
                leg_backward_left.set_angles(kneeForwardAngleLeft, elbowForwardAngleLeft, 0)
                leg_forward_left.set_angles(kneeBackwardAngleRight, elbowBackAngleRight, 0)
            sleep(time_delay)

            current_step = current_step + 1
            if current_step>steps_count:
                break

            #step 2
            if self.right_speed>=0:
                leg_forward_right.set_angles(kneeBackwardAngleRight, elbowBackAngleRight, 0)
                leg_backward_right.set_angles(kneeForwardAngleRight, elbowForwardAngleRight, 0)
            else:
                leg_forward_right.set_angles(kneeForwardAngleRight, elbowBackAngleRight, 0)
                leg_backward_right.set_angles(kneeBackwardAngleLeft, elbowForwardAngleLeft, 0)

            if self.left_speed>=0:
                leg_forward_left.set_angles(kneeForwardAngleLeft, elbowForwardAngleLeft, 0)
                leg_backward_left.set_angles(kneeBackwardAngleLeft, elbowBackAngleLeft, 0)
            else:
                leg_backward_left.set_angles(kneeForwardAngleLeft, elbowBackAngleLeft, 0)
                leg_forward_left.set_angles(kneeBackwardAngleRight, elbowForwardAngleRight, 0)
            sleep(time_delay)

            #step 4
            if self.right_speed>=0:
                leg_forward_right.set_angles(kneeBackwardAngleRight, elbowBackAngleRight, 0)
                leg_backward_right.set_angles(kneeBackwardAngleRight, elbowForwardAngleRight, 0)
            else:
                leg_forward_right.set_angles(kneeBackwardAngleRight, elbowBackAngleRight, 0)
                leg_forward_left.set_angles(kneeForwardAngleLeft, elbowForwardAngleLeft, 0)
            
            if self.left_speed>=0:
                leg_forward_left.set_angles(kneeBackwardAngleLeft, elbowForwardAngleLeft, 0)
                leg_backward_left.set_angles(kneeBackwardAngleLeft, elbowBackAngleLeft, 0)
            else:
                leg_backward_left.set_angles(kneeBackwardAngleLeft, elbowBackAngleLeft, 0)            
                leg_forward_left.set_angles(kneeForwardAngleRight, elbowForwardAngleRight, 0)
            sleep(time_delay)


i2cBus = smbus.SMBus(0)
pca9685 = PCA9685.PCA9685(i2cBus)
dog = Dog()
dog.configure_forward_left_leg(pca9685, PCA9685.CHANNEL00,PCA9685.CHANNEL01,PCA9685.CHANNEL02)
sleep(0.2)
dog.configure_forward_right_leg(pca9685, PCA9685.CHANNEL03,PCA9685.CHANNEL04,PCA9685.CHANNEL05)
sleep(0.2)
dog.configure_backward_left_leg(pca9685,PCA9685.CHANNEL09,PCA9685.CHANNEL10,PCA9685.CHANNEL11)
sleep(0.2)
dog.configure_backward_right_leg(pca9685,PCA9685.CHANNEL06,PCA9685.CHANNEL07,PCA9685.CHANNEL08)
sleep(0.2)
dog.start()
sleep(0.2)
dog.set_speds(1,1)

right_speed = 0
left_speed = 0
steps_count = 0


app = Flask(__name__)
CORS(app)
@app.route('/')
@app.route('/index')
def test():
    #input_text = input('Write right legs speed, left legt speed spetes coutn, for example 1 1 10: ')
    #input_array = input_text.split(' ')
    #if len(input_array) != 3:
    #    print('Wrong command format')
    #    continue
    right_speed = float(request.args.get('right'))
    left_speed = float(request.args.get('left'))
    steps_count = float(request.args.get('steps'))
    dog.set_speds(right_speed, left_speed)
    dog.walk(steps_count)
